import os
import sys
from threading import Timer
import multiprocessing
import traceback
from datetime import datetime, timedelta
from time import sleep, time

from .__task_script_runner import run
from .__util_log import Log
from .__util_task import extend_task_timeout, poll_task, update_task_status, ContainerStoppedException


log = Log({})


def get_run_params(task):
  params = {
    'input': task.get('input'),
    'context_from_arg': task.get('context'),
    'func': task.get('func'),
    'correlation_id': task.get('correlation_id'),
    'func_dir': task.get('func_dir'),
    'store_output': False,
    'storage_type': os.environ.get('TASK_STORAGE_TYPE'),
    'storage_bucket': os.environ.get('TASK_STORAGE_S3FILE_BUCKET'),
    'storage_file_key': os.environ.get('TASK_STORAGE_S3FILE_FILE_KEY'),
    'storage_endpoint': os.environ.get('TASK_STORAGE_S3FILE_ENDPOINT'),
    'artifact_bucket': os.environ.get('ARTIFACT_S3FILE_BUCKET'),
    'artifact_prefix': os.environ.get('ARTIFACT_S3FILE_PREFIX'),
    'artifact_endpoint': os.environ.get('ARTIFACT_S3FILE_ENDPOINT'),
    'artifact_file_key': os.environ.get('ARTIFACT_IDS_SCHEMA_S3FILE_FILE_KEY'),
    'artifact_bucket_private': os.environ.get('ARTIFACT_S3FILE_BUCKET_PRIVATE'),
    'artifact_prefix_private': os.environ.get('ARTIFACT_S3FILE_PREFIX_PRIVATE'),
    'artifact_endpoint_private': os.environ.get('ARTIFACT_S3FILE_ENDPOINT_PRIVATE'),
    'command_endpoint': os.environ.get('COMMAND_ENDPOINT'),
    'fileinfo_endpoint': os.environ.get('FILEINFO_ENDPOINT')
  }

  return params

def healtcheck_worker(run_state):
  run_state['healtcheck_timer'] = None
  task = run_state['task']
  task_process = run_state['task_process']

  if task and task_process:
    task_id = task.get('id')
    try:
      extend_task_timeout(task)
    except:
      log.log(f'Error during timeout extension -> killing task {task_id}')
      task_process.kill()

  healtcheck_timer = Timer(60.0, healtcheck_worker, [run_state])
  run_state['healtcheck_timer'] = healtcheck_timer
  healtcheck_timer.start()


def task_process_fn(task, shared_dict):
  run_params = get_run_params(task)
  sys.path.append(run_params.get('func_dir'))
  try:
    shared_dict['result'] = run(**run_params)
  except:
    e = sys.exc_info()[1]
    log.log(log.generate_error(e))
    shared_dict['error'] = traceback.format_exc()
  sys.path.remove(run_params.get('func_dir'))


def main():

  manager = multiprocessing.Manager()

  shared_dict = manager.dict({'result': None, 'error': None})
  run_state = {'task_process': None, 'task': None, 'healtcheck_timer': None}

  healtcheck_worker(run_state)

  while True:
    task = None

    try:
      task = poll_task()
    except ContainerStoppedException:
      log.log('Container is stopped - exiting...')
      break

    if task:
      task_id = task.get('id')
      log.log(f'Got new task {task_id}')

      shared_dict['result'] = None
      shared_dict['error'] = None

      run_state['task'] = task
      task_process = multiprocessing.Process(name=f'task-{task_id}', target=task_process_fn, args=(task, shared_dict))
      run_state['task_process'] = task_process
      task_process.start()
      task_process.join()

      log.log(f'Task {task_id} process is completed')

      run_state['task_process'] = None
      run_state['task'] = None

      exitcode = task_process.exitcode
      if exitcode != 0:
        if exitcode == -9 or exitcode == 137:
          update_task_status(task, {
            'status': 'failed',
            'result': {
              'error': {
                'message': {
                  'text': f'Invalid exit code {exitcode}',
                  'oomError': True
                }
              }
            }
          })
        else:
          update_task_status(task, {
            'status': 'failed',
            'result': {
              'error': f'Invalid exit code {exitcode}'
            }
          })
        continue

      if shared_dict['result'] != None:
        update_task_status(task, shared_dict['result'])
      else:
        update_task_status(task, {
          'status': 'failed',
          'result': {
            'error': shared_dict['error'] if shared_dict['error'] else 'No content returned by worker'
          }
        })

  if run_state['healtcheck_timer'] and run_state['healtcheck_timer'].is_alive():
    run_state['healtcheck_timer'].cancel()

  return shared_dict

if __name__ == '__main__':
  main()


import json
import sys
import re
import time
import traceback
import requests
from datetime import datetime

from colorama import Fore


def validate_dict_key(d, k):
    if d.get(k) is None:
        print(f'{Fore.RED}{k} is not set!{Fore.RESET}', file=sys.stderr, flush=True)
        return False
    return True

class TsApi:

    def __init__(self, **kwargs):
        res = validate_dict_key(kwargs, 'org')
        res &= validate_dict_key(kwargs, 'api_url')
        res &= validate_dict_key(kwargs, 'auth_token')
        if not res:
            sys.exit(1)
        self.opts = kwargs

    @property
    def __api_url(self):
        return self.opts.get('api_url')

    @property
    def __request_defaults(self):
        return {
            'verify': self.opts.get('ignore_ssl') != True,
            'headers': self.__get_headers()
        }

    def __get_headers(self):
        headers = {'x-org-slug': self.opts.get('org')}
        ts_auth = self.opts.get('auth_token')
        if re.compile(r'^([a-z0-9]+-)+[a-z0-9]+$').match(ts_auth, re.IGNORECASE):
            headers['x-api-key'] = ts_auth
        else:
            headers['ts-auth-token'] = ts_auth
        return headers

    def upload_artifact(self, cfg, artifact_bytes):
        type_to_url = {
            'ids': 'ids',
            'master-script': 'master-scripts',
            'protocol': 'master-scripts',
            'task-script': 'task-scripts'
        }
        url = f'{self.__api_url}/artifact/{type_to_url[cfg.type]}/{cfg.namespace}/{cfg.slug}/{cfg.version}'
        r = requests.post(
            url,
            **self.__request_defaults,
            params={'force': 'true'} if cfg.force else {},
            data=artifact_bytes
        )
        if r.status_code < 400:
            return r.json()
        else:
            print(r.text, file=sys.stderr, flush=True)
            raise Exception(f'HTTP status: {r.status_code}, url: {r.url}')

    def get_task_script_build_info(self, id: str):
        url = f'{self.__api_url}/artifact/builds/{id}'
        r = requests.get(
            url,
            **self.__request_defaults
        )
        if r.status_code < 400:
            return r.json()
        else:
            print(r.text, file=sys.stderr, flush=True)
            raise Exception(f'HTTP status: {r.status_code}, url: {r.url}')

    def get_task_script_build_logs(self, id: str, params):
        url = f'{self.__api_url}/artifact/build-logs/{id}'
        r = requests.get(
            url,
            **self.__request_defaults,
            params={k:v for k,v in params.items() if v is not None}
        )
        if r.status_code < 400:
            return r.json()
        else:
            print(r.text, file=sys.stderr, flush=True)
            raise Exception(f'HTTP status: {r.status_code}, url: {r.url}')



import json
import requests
from time import sleep
from urllib.parse import urlencode

class Fileinfo:

    def __init__(self, endpoint):
        self.endpoint = endpoint

    def ensure_file_exists_in_db(self, file_id, org_slug, check_delay = 2, attempts_max = 10):
        url = f'{self.endpoint}/internal/{org_slug}/file-exists/{file_id}'
        headers = {
            'Content-Type': 'application/json'
        }
        attempt = 0
        while attempt < attempts_max:
            sleep(check_delay)
            response = requests.request('GET', url, headers=headers, verify=False)
            if response.status_code == 200:
                return True
            attempt = attempt + 1
        print({ 'level': 'error', 'message': 'File existence check failed' })
        return False

    def add_labels(self, context_data, file_id, labels, no_propagate = False):
        org_slug = context_data.get('orgSlug')
        self.ensure_file_exists_in_db(file_id, org_slug)

        query_str = urlencode({'noPropagate': 'true'} if no_propagate else {})
        url = f'{self.endpoint}/internal/{org_slug}/files/{file_id}/labels?{query_str}'

        headers = {
            'Content-Type': 'application/json',
            'x-pipeline-id': context_data.get('pipelineId')
        }
        response = requests.request('POST', url, headers=headers, data=json.dumps(labels), verify=False)

        if response.status_code == 200:
            print('Labels successfully added')
            return json.loads(response.text)
        else:
            print('Error adding labels: ' + response.text)
            raise Exception(response.text)

    def get_labels(self, context_data, file_id):
        org_slug = context_data.get('orgSlug')
        self.ensure_file_exists_in_db(file_id, org_slug)

        url = f'{self.endpoint}/internal/{org_slug}/files/{file_id}/labels'
        headers = {
            'Content-Type': 'application/json'
        }
        response = requests.request('GET', url, headers=headers, verify=False)

        if response.status_code == 200:
            print('Labels successfully obtained')
            return json.loads(response.text)
        else:
            print('Error getting labels: ' + response.text)
            raise Exception(response.text)

    def delete_labels(self, context_data, file_id, label_ids):
        org_slug = context_data.get('orgSlug')
        self.ensure_file_exists_in_db(file_id, org_slug)

        suffix = '&'.join(map(lambda id: 'id=' + str(id), label_ids))
        url = f'{self.endpoint}/internal/{org_slug}/files/{file_id}/labels?{suffix}'

        headers = {
            'Content-Type': 'application/json',
            'x-pipeline-id': context_data.get('pipelineId')
        }
        response = requests.request('DELETE', url, headers=headers, verify=False)

        if response.status_code == 200:
            print('Labels successfully deleted')
            return
        else:
            print('Error deleting labels: ' + response.text)
            raise Exception(response.text)



import os
import json
import time
import traceback
from datetime import datetime
from requests import request
from requests.exceptions import RequestException

url = os.environ.get('ORCHESTRATOR_ENDPOINT')
task_group_hash = os.environ.get('TASK_GROUP_HASH')
container_id = os.environ.get('CONTAINER_ID')

def es_datalake_search_eql(payload):
  try:
    search_url = url + '/datalake/searchEql'
    response = request(
      'POST', 
      search_url,
      json=payload,
      headers={
        'x-task-group-hash': task_group_hash,
        'x-container-id': container_id,
        'content-type': 'application/json; charset=utf-8'
      },
      verify=False
    )
    if response.status_code >= 400:
      print({ 'level': 'error', 'message': response.text })
      raise Exception(f'Got {response.status_code} for {search_url}')
    return response.json()
  except RequestException:
      print({ 'level': 'error', 'message': traceback.format_exc() })



import re

meta_name_reg = re.compile('^[0-9a-zA-Z-_+ ]+$')
meta_value_reg = re.compile('^[0-9a-zA-Z-_+., /]+$')
tag_reg = re.compile('^[0-9a-zA-Z-_+. /]+$')
label_name_reg = re.compile('^[0-9a-zA-Z-_+. ]+$')
label_value_reg = re.compile('^[0-9a-zA-Z-_+., /]+$')


def validate_file_meta(meta):
    if meta is None:
        return True
    for k,v in meta.items():
        if not meta_name_reg.match(k):
            raise ValueError(f'Invalid metadata key {k}! Expected pattern: {meta_name_reg.pattern}')
        if not meta_value_reg.match(str(v)):
            raise ValueError(f'Invalid metadata value {v}! Expected pattern: {meta_value_reg.pattern}')
    return True
    

def validate_file_tags(tags):
    if tags is None:
        return True
    for t in tags:
        if not tag_reg.match(str(t)):
            raise ValueError(f'Invalid tag {t}! Expected pattern: {tag_reg.pattern}')
    return True

def validate_file_labels(labels):
    if labels is None:
        return True
    for l in labels:
        n = l.get('name')
        v = str(l.get('value'))
        if not label_name_reg.match(n):
            raise ValueError(f'Invalid label name {n}! Expected pattern: {label_name_reg.pattern}')
        if not label_value_reg.match(v):
            raise ValueError(f'Invalid label value {v}! Expected pattern: {meta_value_reg.pattern}')
    return True


import os
import json
import time
import traceback
from datetime import datetime
from requests import request
from requests.exceptions import RequestException


url = os.environ.get('ORCHESTRATOR_ENDPOINT')
platform_props_hash = os.environ.get('PLATFORM_PROPS_HASH')
task_group_hash = os.environ.get('TASK_GROUP_HASH')
container_id = os.environ.get('CONTAINER_ID')

class ContainerStoppedException(Exception): ...
class TaskUpdateConflictException(Exception): ...

def poll_task():
  try:
    poll_url = url + '/task/poll'
    response = request(
      'POST', 
      poll_url,
      json={
        'platformPropsHash' : platform_props_hash,
        'taskGroupHash' : task_group_hash,
        'containerId' : container_id
      },
      verify=False
    )
    if response.status_code >= 400:
      message = f'Got {response.status_code} for {poll_url}'
      print({ 'level': 'error', 'message': message })
      if response.status_code == 409:
        raise ContainerStoppedException(message)
      time.sleep(2)
      return None
    return generate_task_from_reponse(response.json())
  except RequestException:
      print({ 'level': 'error', 'message': traceback.format_exc() })

def update_task_status(task, result):
  try:
    task_id = task.get('id')
    update_url = url + f'/task/{task_id}/update-status'
    response = request(
      'POST', 
      update_url,
      json=result,
      verify=False
    )
    if response.status_code >= 400:
      message = f'Got {response.status_code} for {update_url}'
      print({ 'level': 'error', 'message': message })
  except RequestException:
      print({ 'level': 'error', 'message': traceback.format_exc() })

def generate_task_from_reponse(body):
  if body:
    data = body.get('data')
    return {
      'id': body.get('id'),
      'context': data.get('context', {}) or {},
      'input': data.get('input', {}) or {},
      'secrets': data.get('secrets', {}) or {},
      'func': data.get('func'),
      'workflow_id': data.get('workflowId'),
      'correlation_id': body.get('correlationId') or body.get('id'),
      'func_dir': data.get('funcDir', './func') or './func'
    }
  
  return {}

def extend_task_timeout(task):
  try:
    task_id = task.get('id')
    extend_timeout_url = url + f'/task/{task_id}/extend-timeout'

    response = request(
      'POST', 
      extend_timeout_url,
      json={},
      verify=False
    )
    if response.status_code >= 400:
      message = f'Got {response.status_code} for {extend_timeout_url}'
      print({ 'level': 'error', 'message': message })
      if response.status_code == 409:
        raise TaskUpdateConflictException(message)
    print({ 'level': 'debug', 'message': f'EXTENDING: {task_id}' })
  except RequestException:
      print({ 'level': 'error', 'message': traceback.format_exc() })
